extends RigidBody

export var manic_color = Color()
export var depressive_color = Color()
export var mass = 1.0

var saved_mode = "manic"

var is_goal = false

onready var material = get_node("MeshInstance").get_material_override().duplicate()

onready var init_transform = Transform( get_transform() )

func _ready():
	get_node("/root/Scene").connect("mode_changed", self, "change_mode")
	add_to_group("Objects")
	
	get_node("MeshInstance").set_material_override( material )
	material.set_parameter( FixedMaterial.PARAM_DIFFUSE, manic_color )
	
	set_mode( RigidBody.MODE_KINEMATIC )
	
	set_mass(mass)
	
#	get_node("MeshInstance").set_cast_shadows_setting( GeometryInstance.SHADOW_CASTING_SETTING_OFF )
	
#	set_fixed_process( true )

func _fixed_process(delta):
	if get_mode() == RigidBody.MODE_KINEMATIC and get_transform() != init_transform :
		set_transform( init_transform )
		set_fixed_process(false)
	
func reset():
	set_mode( RigidBody.MODE_KINEMATIC )
	set_fixed_process(true)
	
	var timer = Timer.new()
	timer.set_wait_time(1)
	add_child( timer )
	timer.start()
	
	yield(timer, "timeout")
	timer.queue_free()
	if saved_mode == "depressive":
		set_mode( RigidBody.MODE_RIGID )
		
		var temp_mass = mass * 100
		
		var tween = Tween.new()
		tween.interpolate_method(self, "set_mass", temp_mass, get_mass(), 1, Tween.TRANS_LINEAR, Tween.EASE_IN)
		add_child(tween)
		tween.start()
		
		yield(tween, "tween_complete")
		tween.queue_free()
	
func material_change(color):
	material.set_parameter( FixedMaterial.PARAM_DIFFUSE, color )

func change_mode(mode):
	saved_mode = mode
	
	var color
	if mode == "manic":
		color = manic_color
	else:
		color = depressive_color
	
	
	var tween = Tween.new()
	tween.interpolate_method( self, "material_change", material.get_parameter( FixedMaterial.PARAM_DIFFUSE ), color, 0.4, Tween.TRANS_SINE, Tween.EASE_OUT_IN)
	add_child(tween)
	tween.start()
	
	yield(tween, "tween_complete")
	tween.queue_free()
	
func make_goal():
	is_goal = true
	add_child( preload("res://Goal.tscn").instance() )
	
	var tween = Tween.new()
	tween.interpolate_method( self, "change_emission", 0, 1, 1, Tween.TRANS_CIRC, Tween.EASE_IN )
	tween.set_repeat( true )
	add_child( tween )
	tween.start()
	
func change_emission(val):
	get_node("MeshInstance").get_material_override().set_parameter( FixedMaterial.PARAM_EMISSION, Color(val,val,val) )